package com.fusionv.linkv;

import android.app.Activity;
import android.content.Context;
import android.util.Log;

import com.linkv.rtc.LVConstants;
import com.linkv.rtc.LVErrorCode;
import com.linkv.rtc.LVRTCEngine;
import com.linkv.rtc.beauty.LVBeautyManager;
import com.linkv.rtc.callback.LVRTCCallback;
import com.linkv.rtc.callback.LVResultCallback1;
import com.linkv.rtc.callback.LVResultCallback2;
import com.linkv.rtc.entity.LVAVConfig;
import com.linkv.rtc.entity.LVAudioVolume;
import com.linkv.rtc.entity.LVUser;
import com.linkv.rtc.entity.LVVideoStatistic;

import java.math.BigDecimal;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** LinkvPlugin */
public class LVRTCEnginePlugin implements MethodCallHandler {

  private Registrar mRegistrar;
  public static LVRTCEngine mRtcEngine;

  public static Activity mContext;
  private String TAG = "Wing";
  private static   MethodChannel _channel;

  /** Plugin registration. */
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "LVRTCEngine");
    channel.setMethodCallHandler(new LVRTCEnginePlugin(registrar));
    _channel = channel;

    registrar.platformViewRegistry().registerViewFactory("surface", new FlutterSurfaceViewFactory(registrar.messenger()));
  }

  public LVRTCEnginePlugin(Registrar registrar) {
    this.mRegistrar = registrar;
  }

  @Override
  public void onMethodCall(MethodCall call, Result result) {
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    } else if (call.method.equals("initSDK")) {
      initSDK(call, result);
    } else if (call.method.equals("unInitSDK")) {
      unInitSDK(call, result);
    } else if (call.method.equals("loginRoom")) {
      loginRoom(call, result);
    } else if (call.method.equals("logoutRoom")) {
      logoutRoom(result);
    } else if (call.method.equals("setPublishQualityMonitorCycle")) {
      setPublishQualityMonitorCycle(call, result);
    } else if (call.method.equals("startPublishing")) {
      startPublishing(call, result);
    } else if (call.method.equals("getSdkVersion")) {
      result.success(LVRTCEngine.buildVersion());
    } else if (call.method.equals("startCapture")) {
      startCapture(result);
    } else if (call.method.equals("stopCapture")) {
      stopCapture(result);
    } else if (call.method.equals("stopPublishStream")) {
      stopPublishStream(result);
    } else if (call.method.equals("startPlayingStream")) {
      startPlayingStream(call, result);
    } else if (call.method.equals("stopPlayingStream")) {
      stopPlayingStream(call, result);
    } else if (call.method.equals("setPlayQualityMonitor")) {
      setPlayQualityMonitor(call, result);
    } else  if (call.method.equals("switchCamera")) {
        switchCamera(call, result);
    } else  if (call.method.equals("setAVConfig")) {
      setAVConfig(call, result);
    } else  if (call.method.equals("setBeautyLevel")) {
      setBeautyLevel(call, result);
    }
    else {
      result.notImplemented();
    }
  }

  private void setBeautyLevel(final MethodCall call, final Result result) {
    int beauty = call.argument("beauty");
    double level_d = call.argument("level");
    BigDecimal decimal = new BigDecimal(level_d);
    float level = decimal.floatValue();
    switch (beauty) {
      case 0:
        mRtcEngine.getLVBeautyManager().setBeautyLevel(level);
        break;
      case 1:
        mRtcEngine.getLVBeautyManager().setBrightLevel(level);
        break;
      case 2:
        mRtcEngine.getLVBeautyManager().setToneLevel(level);
        break;
      default:
        break;
    }
  }

  private void setAVConfig(final MethodCall call, final Result result) {
    int config = call.argument("config");
    LVAVConfig avConfig = new LVAVConfig(config);
    mRtcEngine.setAVConfig(avConfig);

    Log.d(TAG, "setAVConfig config = " + config);
    result.success(true);
  }

  private  void switchCamera(final MethodCall call, final Result result) {
    int position = call.argument("cameraPosition");
    if ( position == 0) {
      mRtcEngine.switchCamera(LVConstants.CMRTCCameraPosition.BACK);
    } else {
      mRtcEngine.switchCamera(LVConstants.CMRTCCameraPosition.FRONT);
    }
    result.success(true);
  }

  private void setPlayQualityMonitor(final MethodCall call, final Result result) {
    Log.d(TAG, "setPlayQualityMonitor seconds = " + call.argument("seconds"));
    mRtcEngine.setPlayQualityMonitorCycle((int)call.argument("seconds"));
    result.success(true);
  }

  private void stopPlayingStream(final MethodCall call, final Result result) {
    String streamId = call.argument("streamId");
    mRtcEngine.stopPlayingStream(streamId);
    result.success(true);
  }

  private void startPlayingStream(final MethodCall call, final Result result) {
      String streamId = call.argument("streamId");
      Log.d(TAG, "startPlayingStream streamId = " + streamId);
      mRtcEngine.startPlayingStream(streamId);
      result.success(true);
  }

  private void stopPublishStream(final Result result) {
    mRtcEngine.stopPublishing();
    result.success(true);
  }

  private void stopCapture(final  Result result) {
    mRtcEngine.stopCapture();
    result.success(true);
  }

  private void startCapture(final  Result result) {
    mRtcEngine.startCapture();
    result.success(true);
  }

  private void logoutRoom(final Result result) {
    Log.d(TAG, "logout start");
    mRtcEngine.logoutRoom(new LVResultCallback1<Integer>() {
      @Override
      public void onResult(final Integer integer) {
        Log.d("Wing", "logout end pid = " + Thread.currentThread().getId());
//        mContext.runOnUiThread(new Runnable() {
//          @Override
//          public void run() {
//            result.success(integer);
//          }
//        });
      }
    });
  }

  public void initSDK(final MethodCall call, final MethodChannel.Result result) {

    if(mRtcEngine == null) {
      LVRTCEngine.setUseTestEnv((boolean)call.argument("isTestEnv"));
      LVRTCEngine.setUseInternationalEnv((boolean)call.argument("isInternationalEnv"));
      mRtcEngine = LVRTCEngine.getInstance(mRegistrar.context());
      mRtcEngine.initSDK();
      mRtcEngine.setAVConfig(new LVAVConfig(LVAVConfig.PROFILE_360P));
    }

    String appId = call.argument("AppId");
    String appSign = call.argument("AppSign");
    mRtcEngine.auth(appId, appSign, "", new LVResultCallback1<Integer>(){
      @Override
      public void onResult(Integer integer) {
        result.success(integer);
      }
    });

    Log.d(TAG, "initSDK");
  }

  public void unInitSDK(final MethodCall call, final MethodChannel.Result result) {
    if(mRtcEngine != null)
      mRtcEngine.unInitSDK();
    mRtcEngine = null;
  }


  public void loginRoom(final MethodCall call, final MethodChannel.Result result) {
    Log.d("Wing", "login room start");
    mRtcEngine.loginRoom((String) call.argument("roomId"), (String)call.argument("userId"), (boolean)call.argument("isHost"), (boolean)call.argument("isAudioOnly"),  new LVResultCallback2<Integer, ArrayList<LVUser>>(){
      @Override
      public void onResult(final Integer code, final ArrayList<LVUser> lvUsers) {
        final ArrayList<Map<String, Object>> users = new ArrayList<Map<String, Object>>();
        if (code == LVErrorCode.SUCCESS) {
          for (LVUser user:lvUsers) {
            Map<String, Object> userinfo = new HashMap<>();
            userinfo.put("userId", user.userId);

            ArrayList<String> pullUrls = new ArrayList<String>();
            if(user.pullUrls != null) {
              for(String url : user.pullUrls) {
                pullUrls.add(url);
              }
            }
            userinfo.put("pullUrls", pullUrls);
            users.add(userinfo);
          }
          setVideoCallback();
        }
        Log.d("Wing", "login room end");
        result.success(new HashMap(){
          {put("code", code);
            put("users", users);
          }
        });
      }
    });
  }

  public void setPublishQualityMonitorCycle(final MethodCall call, final MethodChannel.Result result) {
    Log.d(TAG, "setPublishQualityMonitorCycle");
    mRtcEngine.setPublishQualityMonitorCycle((int)call.argument("seconds"));
    result.success(true);
  }
  public void startPublishing(final MethodCall call, final MethodChannel.Result result) {
    Log.d(TAG, "startPublishing");
    mRtcEngine.startPublishing();
    result.success(true);
  }

  private void setVideoCallback() {
    mRtcEngine.setLiveRoomCallback(new LVRTCCallback() {
      @Override
      public long onDrawFrame(ByteBuffer byteBuffer, int i, int i1, int i2, String s, String s1) {
        return 0;
      }

      @Override
      public void onAddRemoter(LVUser lvUser) {
        final Map<String, Object> userinfo = new HashMap<>();
        userinfo.put("userId", lvUser.userId);

        ArrayList<String> pullUrls = new ArrayList<String>();
        if(lvUser.pullUrls != null) {
          for(String url : lvUser.pullUrls) {
            pullUrls.add(url);
          }
        }
        userinfo.put("pullUrls", pullUrls);

        Log.d(TAG, "onAddRemoter Android user_id = " + lvUser.userId);
        mContext.runOnUiThread(new Runnable() {
          @Override
          public void run() {
            _channel.invokeMethod("onAddRemoter", userinfo);
          }
        });
      }

      @Override
      public void onDeleteRemoter(String s) {
        final HashMap map = new HashMap();
        map.put("streamId", s);

        Log.d(TAG, "onDeleteRemoter streamId = " + s);
        mContext.runOnUiThread(new Runnable() {
          @Override
          public void run() {
            _channel.invokeMethod("onDeleteRemoter", map);
          }
        });
      }

      @Override
      public void onKickOff(int reason, String s) {
        final HashMap map = new HashMap();
        map.put("reason", reason);
        map.put("roomId", s);
        mContext.runOnUiThread(new Runnable() {
          @Override
          public void run() {
            _channel.invokeMethod("onKickOff", map);
          }
        });
      }

      @Override
      public void onRoomDisconnected(int code) {
        final HashMap map = new HashMap();
        map.put("code", code);
        mContext.runOnUiThread(new Runnable() {
          @Override
          public void run() {
            _channel.invokeMethod("onRoomDisconnect", map);
          }
        });
      }

      @Override
      public void onRoomReconnected() {
        mContext.runOnUiThread(new Runnable() {
          @Override
          public void run() {
            _channel.invokeMethod("onRoomReconnect", null);
          }
        });
      }

      @Override
      public void onMixComplete(boolean b) {
        final HashMap map = new HashMap();
        map.put("b", b);
        mContext.runOnUiThread(new Runnable() {
          @Override
          public void run() {
            _channel.invokeMethod("onMixComplete", map);
          }
        });
      }

      @Override
      public void onPublishQualityUpdate(LVVideoStatistic lvVideoStatistic) {
        Log.d("Wing", "onPublishQualityUpdate fps = " + lvVideoStatistic.videoFps);

        final  HashMap map = mapFromVideoStatistic(lvVideoStatistic);

        mContext.runOnUiThread(new Runnable() {
          @Override
          public void run() {
            _channel.invokeMethod("onPublishQualityUpdate", map);
          }
        });
      }

      @Override
      public void onPublishStateUpdate(int i) {
        final HashMap map = new HashMap();
        map.put("state", i);

        mContext.runOnUiThread(new Runnable() {
          @Override
          public void run() {
            _channel.invokeMethod("onPublishStateUpdate", map);
          }
        });
      }

      @Override
      public void onPlayQualityUpdate(LVVideoStatistic lvVideoStatistic, String s) {
        Log.d("Wing", "onPlayQualityUpdate fps = " + lvVideoStatistic.videoFps);

        final  HashMap map = mapFromVideoStatistic(lvVideoStatistic);
        map.put("uid", s);
        mContext.runOnUiThread(new Runnable() {
          @Override
          public void run() {
            _channel.invokeMethod("onPlayQualityUpdate", map);
          }
        });
      }

      @Override
      public void onPlayStateUpdate(int i, String s) {
        Log.d("Wing", "onPlayStateUpdate state = " + i);

        final HashMap map = new HashMap();
        map.put("state", i);
        map.put("streamId", s);

        mContext.runOnUiThread(new Runnable() {
          @Override
          public void run() {
            _channel.invokeMethod("onPlayStateUpdate", map);
          }
        });
      }

      @Override
      public void onAudioMixStream(byte[] bytes, int i, int i1, int i2, int i3, LVConstants.AudioRecordType audioRecordType) {

      }

      @Override
      public void onAudioVolumeUpdate(ArrayList<LVAudioVolume> arrayList) {

      }

      @Override
      public void onExitRoomComplete() {
        mContext.runOnUiThread(new Runnable() {
          @Override
          public void run() {
            _channel.invokeMethod("onExitRoomComplete", null);
          }
        });
      }

      @Override
      public String onMediaSideInfoInPublishVideoFrame() {
        return null;
      }
    });
  }

  HashMap mapFromVideoStatistic(LVVideoStatistic lvVideoStatistic) {
    HashMap map = new HashMap();
    map.put("videoSentPackets", lvVideoStatistic.videoSentPackets);
    map.put("videosentKbytes", lvVideoStatistic.videoSentKBytes);
    map.put("videoFps", lvVideoStatistic.videoFps);
    map.put("videoLostPackets", lvVideoStatistic.videoLostPackets);
    map.put("videoBitrateBps", lvVideoStatistic.videoBitratebps);
    map.put("videoRtt", lvVideoStatistic.videoRtt);
    map.put("videoLostPercent", lvVideoStatistic.videoLostPercent);
    map.put("frameWidth", lvVideoStatistic.frameWidth);
    map.put("frameHeight", lvVideoStatistic.frameHeight);
    return map;
  }

}
